package outils;

public class Individual {

// An "Individual" is one of the elements of the spatialized sample of the metacommunity. 
// It is caracterized by a geographic position, a location of sampling, a species, a number within the sample of individuals, a time (=time when we learned the identity of its mainland ancestor).
// It may pertain to a lineage and may be a "topInd" = the upper known mother of a lineage.

// ATTRIBUTES:

	private int loc;                // Location of the individual (index of the island patch)
	private int spc;				// Species attributed to the individual, which is only determined during the coalescence process. Its value by default (-1) is then replaced by a positive number.			
	private int number;				// Each individual has a number
	private int time;				// We keep memory of the time when we learned the mainland ancestor of each individual (useful for the algorithm, especially for writing the text file which keep in memory the history of coalescence)
	private Lineage lign;			// Its lineage (useful for the algorithm), double chain
	int topInd;						// -1 if the individual is not a top individual (= the upper known mother of a lineage) and 1 if it is a top individual
					
	
// CONSTRUCTOR:

	public Individual(int p, int nb){

		spc=-1;
		loc=p;						// initial parameter
		time=-1;
		number=nb;					// initial parameter, different for each individual
		topInd=1;					// At the beginning each individual is the "topInd" of the lineage containing only itself.
		
	}

// METHODS:

	public Lineage getLineage(){
		return lign;
	}

	public int getLoc(){
		return loc;
	}

	public int getSpc(){
		return spc;
	}
	
	public int getTime(){
		return time;
	}

	public int getNum(){
		return number;
	}
	
	public int getTopInd(){
		return topInd;
	}
	
	public void setLoc(int p){
		loc = p;
	}

	public void setSpc(int e){
		spc = e;
	}

	public void setTime(int t){
		time = t;
	}
	
	public void setNum(int n){
		number = n;
	}
	
	public void setTopInd(int t){
		topInd = t;
	}
	
	public void setLineage(Lineage l){
		lign = l; 						//Changing the pointer
	}
	
	public String sumUp(){
		StringBuffer s = new StringBuffer();
		s.append("Species ");
		s.append(spc);
		s.append(" in patch ");
		s.append(loc);
		return s.toString();
	}

// TEST:

	public static void main(String args[]){
		Individual i = new Individual(1,0);       	// localisation = 1, number = 0
		System.out.println(i.sumUp());
		i.setSpc(2);                        		// species=2
		System.out.println(i.sumUp());
	}

}
