package outils;

import java.util.Iterator;
import java.util.LinkedList;

public class Lineage {

//	A "Lineage" is a genealogical structure which links a group of individuals "ind_ech", which is included into the sample at present time,
//	and an individual from the past, which is the common ancestor of this group.

//	ATTRIBUTES:

		private LinkedList<Individual> ind_sampl; 		 // List of the individuals of the sample at present time on which the lineage is built 
		private Individual ancestor;                	 // Common ancestor of ind_ech
		
//	CONSTRUCTOR:

		public Lineage(Individual ind){
			ind_sampl = new LinkedList<Individual>();
			ind_sampl.add(ind);	// Changing pointers, not values; Each lineage is set at T=0 on 1 individual sampled
			ancestor = ind;		// idem
			ind.setLineage(this);
		}

//	METHODS:

		public void mother(LinkedList<Individual> list){
			ind_sampl.addAll(list); 	//Changing pointers, not values;
										//This method allow absorbing other lineages within the actual lineage. The absorbed lineage is not traced anymore. 
										//The reverse pointer of the absorbed individuals does not need to be updated because it is never used afterwards. 
		}

		public void descendFrom(Individual i){
			ancestor = i;	//Changing pointers, not values;
							//Method used to trace the geographic movements of the ancestor as we go back in the past. 
							//No influence on the support of the lineage, which only changes during coalescences.
			i.setLineage(this);
		}
		
		public void newMainlandSpecies(Individual i, int[] distri){
			double r = Math.random();
			int numSpecies=0;
			int r2 = (int)Math.floor(r*distri[distri.length-1]+1);	//Integer between [1,J+1[
			for (int i1=0; i1 < distri.length; i1++){
				if (distri[i1]>r2){
					numSpecies = i1+1;
					break;
				}
			}
			i.setSpc(numSpecies); //Changing pointers, not values;
							//This method is done to trace the geographic movements of the ancestor as we go back in the past.
							//But it is used only when the ancestor is absorbed as a new species; the lineage stops being traced.
							//No influence on the support of the lineage, which only changes during coalescences.
			ancestor = i;
			i.setLineage(this);
			Iterator<Individual> it = ind_sampl.iterator();
			while(it.hasNext()){
				it.next().setSpc(numSpecies);
			}
		}

		public LinkedList<Individual> getSampl(){
			return(ind_sampl); 	//Changing pointers, not values;
		}

		public Individual getAnc(){
			return(ancestor);
		}

		public void set(int e){
			Iterator<Individual> it = ind_sampl.iterator();
			while(it.hasNext()){
				it.next().setSpc(e);
			}
		}
		
		public void adjustTime(int t){
			Iterator<Individual> it = ind_sampl.iterator();
			while(it.hasNext()){
				it.next().setTime(t);
			}
		}

// I/O:

		public String sumUp(){
			StringBuffer s = new StringBuffer();
			s.append("Ancestor : ");
			s.append(ancestor.sumUp());
			s.append("\nSupport within the sample :\n");
			Iterator<Individual> it = ind_sampl.iterator();
			while(it.hasNext()){
				s.append("   "+it.next().sumUp()+"\n");
			}
			return s.toString();
		}

// TEST:
		public static void main(String args[]){
			Individual i = new Individual(1,0);
			i.setSpc(1);
			Lineage l = new Lineage(i);
			System.out.println(l.sumUp());
			i.setSpc(2);
			System.out.println(l.sumUp());
			Individual j = new Individual(2,1);
			j.setSpc(3);
			Lineage m = new Lineage(j);
			l.mother(m.getSampl());
			System.out.println(l.sumUp());
			l.set(4);
			System.out.println(l.sumUp());
		}
}
