package outils;

import java.io.*;
import java.util.StringTokenizer;

//This class allow:
//- Reading the matrix of connectivity (double, sep=" ")
	// This file, "Mconnectivity_unstruct_test.txt" = matrix M of size [number of archipelagos + 1(mainland)]*2, 
	// with M[i,0]= probability for a vacant site in archipelago i to be colonized by a propagule coming from within the archipelago, and M[i,1]= probability for a vacant site in i to be colonized by a propagule coming from the mainland.
	// The sum of all values in each row must be 1 (probability).
//- Reading the vector of the mainland cumulative species abundance distribution  (integers, sep =";", in line)
	// This file, "DistriMainland_test.txt" = (number of species; cumulative abundance of each mainland species...); eg: (3; 7000; 9000; 10000) = 10,000 individuals on the mainland, distributed in 3; species 1 = 70%, species 2 = 20% and species 3 = 10%.
//- Reading the vector of the carrying capacity of each archipelago (integers, sep =";", in line)
	// This file, "CarryingCapacities_unstruct_test.txt" = (number of archipelagos; number of individuals on each archipelago...); eg: (2; 90; 90) = 2 archipelagos, each with 90 individuals.
//- Reading the vector of the sampling frequency (sampling effort) within each archipelago (double, sep =";", in line)
	// This file, "SamplingFreq_unstruct_test.txt" = (number of archipelagos; sampling frequency on each archipelago...); eg: (3; 0.1; 0.1) = 2 archipelagos, with 10% of the individuals sampled on each archipelago (hence in this example 9 individuals/archipelago sampled and followed in the coalescence)
//- Writing, for each archipelago ARCHIP, a table summarizing all the information needed to know the complete history of coalescence of all simulation replicates: initial state, final state and all the events along the coalescence.
	// This file, "coaHist_unstruct_test_ARCHIP.txt" (coalescence history) = table of strings, 8 columns:
		// - Time step (present time = 0)
		// - Individual of interest (target individual during the coalescence) + if this individual is a TopInd, its descendants (index-index-index-...); Each individual is caracterized by its index.
		// - Location of this individual of interest (archipelago index, at this time step)
		// - Species if its mainland ancestor; -1 if unknown at this time step.
		// - Event which happened: 0 if local recruitment, 1 if external recruitment, 2 if coalescence (local or external), and 3 if migration from the mainland.
		// - From = index of the target individual (if coalescence, = which individual just joined the lineage; AND if migration from the mainland, = the topIndividual of the lineage); 0 if local recruitment.
		// - Position = location (archipelago index) of the target individual (before coalescence), if local or external coalescence. 0 if other event.
		// - Number of active individuals at this time step (used in the next step, "spc_struc", to calculate the time lag between each event).
	// This file is read and used in the next step, "spc_unstruct" (which implements speciation, working forward in time), to assign species to the individuals at present time.
	// Unlike "coa_struct", this file contains the coalescence histories of all simulation replicates SIM, for archipelago ARCHIP (in "coa_struct", the equivalent file contains only 1 simulation replicate)
	// Unlike "coa_struct", there is no external recruitement, and no external coalescence; and, because we consider only one unstructured archipelago at a time, its index is always zero.

public class ReadTable{	
	
	public static double [][] readConnectivity() throws IOException{										// Reading a table of doubles (matrix of connectivity)
		BufferedReader connectivitySize = new BufferedReader(new FileReader("Mconnectivity_unstruct_test.txt"));		// First, we look for the size of the matrix, to identify the number of archipelagos
		String str3 = connectivitySize.readLine();
		int NM = 0;
		while(str3 != null){
			str3 = connectivitySize.readLine();
			NM++;
		}					// Be careful: NM = number of archipelagos + mainland (Number of rows of the matrix of connectivity)
		
		BufferedReader connectivity = new BufferedReader(new FileReader("Mconnectivity_unstruct_test.txt"));			// Secondly, we read this matrix and write it into Mcon, used by Coalescent.java	
		String matcon[][];
		matcon = new String[NM][2];
		String str = connectivity.readLine();						// str = one line of the table
		int i1 = 0, j1 = 0;
		while(str != null){						
			StringTokenizer st = new StringTokenizer(str," ");		// st = one word of the line
			for (j1=0; j1<2; j1++){
				matcon[i1][j1] = st.nextToken();
			}
			str = connectivity.readLine();
			i1++;
		}
		double Mcon[][];											// matcon is a table of strings: it must be transformed into a table of doubles, = Mcon
		Mcon = new double[NM][2];
		for (i1=0; i1<NM; i1++){
			for (j1=0; j1<2; j1++){
				Mcon[i1][j1] = Double.parseDouble(matcon[i1][j1]);
			}													
		}
		return Mcon;
	}
	
	public static int[] readDistri() throws IOException{													// Reading the vector of the mainland species abundance distribution  (integers, sep =";")
		BufferedReader distribution = new BufferedReader(new FileReader("DistriMainland_test.txt")); 		
		String str2 = distribution.readLine();					
		StringTokenizer st2 = new StringTokenizer(str2,";");		// str = the complete line of the "table" = the vector
		String nbData;
		nbData = st2.nextToken();									// 1st word = number of species
		int nb = Integer.parseInt(nbData);	 
		String distri1[];
		distri1 = new String[nb];
		int i2 = 0, j2 = 0;
		while(str2 != null){
			while (j2 < nb){
				distri1[j2] = st2.nextToken();
				j2++;
			}
			str2 = distribution.readLine();
		}
		int[] distri;												// distri1 is a table of strings: it must be transformed into a table of doubles, = distri
		distri = new int[nb];
		for (i2=0; i2<nb; i2++){
			distri[i2] = Integer.parseInt(distri1[i2]);
		}
		return distri;
	}
	
	public static int[] readK() throws IOException{															// Reading the vector of the carrying capacities of the archipelagos (integers, sep =";")
		BufferedReader CCPatches = new BufferedReader(new FileReader("CarryingCapacities_unstruct_test.txt")); 
		String str3 = CCPatches.readLine();					
		StringTokenizer st3 = new StringTokenizer(str3,";");		// str2 = = the complete line of the "table" = the vector
		String nbData;
		nbData = st3.nextToken();									// 1st word = number of archipelagos
		int nb = Integer.parseInt(nbData);
		String[] CCPat1 = new String[nb];
		int i3 = 0, j3 = 0;
		while(str3 != null){
			while (j3 < nb){
				CCPat1[j3] = st3.nextToken();
				j3++;
			}
			str3 = CCPatches.readLine();
		}
		int[] CCPat;												// CCPat1 is a table of strings: it must be transformed into a table of doubles, = CCPat
		CCPat = new int[nb];
		for (i3=0; i3<nb; i3++){
			CCPat[i3] = Integer.parseInt(CCPat1[i3]);
		}
		return CCPat;
	}
	
	public static double[] readSamplingFreq() throws IOException{											//Reading the vector of the frequencies of sampling within each archipelago (double, sep =";")
		BufferedReader SFPatches = new BufferedReader(new FileReader("SamplingFreq_unstruct_test.txt")); 
		String str3 = SFPatches.readLine();					
		StringTokenizer st3 = new StringTokenizer(str3,";");		// str2 = the complete line of the "table" = the vector
		String nbData;
		nbData = st3.nextToken();									// 1st word = number of archipelagos
		int nb = Integer.parseInt(nbData);
		String[] freq1 = new String[nb];
		int i3 = 0, j3 = 0;
		while(str3 != null){
			while (j3 < nb){
				freq1[j3] = st3.nextToken();
				j3++;
			}
			str3 = SFPatches.readLine();
		}
		double[] freq;												// freq1 is a table of strings: it must be transformed into a table of doubles, = freq
		freq = new double[nb];
		for (i3=0; i3<nb; i3++){
			freq[i3] = Double.parseDouble(freq1[i3]);			
		}
		return freq;
	}

	public static void writeOutput(Individual[] sample, Individual target, int motherNum, int time, int event,int from, int position, int nbActiv, int archip) { 	// Writing the table summarizing all the information needed to know the complete history of coalescence: initial + intermediate states.
		try{
			FileWriter fw = new FileWriter("coaHist_unstruct_test_"+archip+".txt",true);
			BufferedWriter bw = new BufferedWriter(fw);
			for (int i=0; i<sample.length; i++){														// for all the active individuals (=topInd),
				if (sample[i].getTopInd()==1){															
					if (sample[i].getSpc() == -1 | sample[i].getTime() == time){								
						if (time == 0){																	
							String Time = Integer.toString(time);
							bw.write(Time);
							bw.write(' ');
							for (int i1=0; i1<sample[i].getLineage().getSampl().size();i1++){
								String Num = Integer.toString(sample[i].getLineage().getSampl().get(i1).getNum());
								bw.write(Num);
								bw.write('-');
							}
							bw.write(' ');
							String Loc = Integer.toString(sample[i].getLoc());
							bw.write(Loc);
							bw.write(' ');
							String Esp = Integer.toString(sample[i].getSpc());
							bw.write(Esp);
							bw.write(' ');
							bw.write('0');
							bw.write(' ');
							bw.write('0');
							bw.write(' ');
							bw.write('0');
							bw.write(' ');
							String Act = Integer.toString(nbActiv+1);
							bw.write(Act);
							bw.newLine();
						}
						else if (sample[i].getNum() == target.getNum()){										// - target individual
							String Time = Integer.toString(time);
							bw.write(Time);
							bw.write(' ');
							for (int i1=0; i1<sample[i].getLineage().getSampl().size();i1++){
								String Num = Integer.toString(sample[i].getLineage().getSampl().get(i1).getNum());
								bw.write(Num);
								bw.write('-');
							}
							bw.write(' ');
							String Loc = Integer.toString(sample[i].getLoc());
							bw.write(Loc);
							bw.write(' ');
							String Esp = Integer.toString(sample[i].getSpc());
							bw.write(Esp);
							bw.write(' ');
							String Event = Integer.toString(event);
							bw.write(Event);	
							bw.write(' ');
							String From = Integer.toString(from);
							bw.write(From);
							bw.write(' ');
							bw.write('0');
							bw.write(' ');
							String Act = Integer.toString(nbActiv+1);
							bw.write(Act);
							bw.newLine();
						}
						else if (event == 2 & sample[i].getNum() == motherNum){									// - mother during an event of coalescence (otherwise events 2 are not written because the target individual is not a topInd anymore)
							String Time = Integer.toString(time);
							bw.write(Time);
							bw.write(' ');
							for (int i1=0; i1<sample[i].getLineage().getSampl().size();i1++){
								String Num = Integer.toString(sample[i].getLineage().getSampl().get(i1).getNum());
								bw.write(Num);
								bw.write('-');
							}
							bw.write(' ');
							String Loc = Integer.toString(sample[i].getLoc());
							bw.write(Loc);
							bw.write(' ');
							String Esp = Integer.toString(sample[i].getSpc());
							bw.write(Esp);
							bw.write(' ');
							String Event = Integer.toString(event);
							bw.write(Event);	
							bw.write(' ');
							String From = Integer.toString(from);
							bw.write(From);
							bw.write(' ');
							String Position = Integer.toString(position);
							bw.write(Position);
							bw.write(' ');
							String Act = Integer.toString(nbActiv+1);
							bw.write(Act);
							bw.newLine();
						}
					}
				}
			}
			bw.close();
			fw.close();
		} catch (IOException e){
			System.out.println("Error when writing the file coaHist.txt!");
		}
	}

	public static void writeLastOutput(Individual[] sample, Individual target, int time, int event,int from, int archip) {			// Writing the table summarizing all the information needed to know the complete history of coalescence: final state.
		try{
			FileWriter fw = new FileWriter("coaHist_unstruct_test_"+archip+".txt",true);
			BufferedWriter bw = new BufferedWriter(fw);			
			for (int i=0; i<sample.length; i++){								// For each individual in the sample, write a line of information (allow knowing the last event in the coalescence process, as well as the species of their mainland ancestor)
					String Time = Integer.toString(time);
					bw.write(Time);
					bw.write(' ');
					for (int i1=0; i1<sample[i].getLineage().getSampl().size();i1++){
						String Num = Integer.toString(sample[i].getLineage().getSampl().get(i1).getNum());
						bw.write(Num);
						bw.write('-');
					}
					bw.write(' ');
					String Loc = Integer.toString(sample[i].getLoc());
					bw.write(Loc);
					bw.write(' ');
					String Esp = Integer.toString(sample[i].getSpc());
					bw.write(Esp);
					bw.write(' ');
					if (sample[i].getNum() == target.getNum()){
						String Event = Integer.toString(event);
						bw.write(Event);
						bw.write(' ');
						String From = Integer.toString(from);
						bw.write(From);
						bw.write(' ');
						bw.write('0');
					}
					else {
						bw.write('0');
						bw.write(' ');
						bw.write('0');
						bw.write(' ');
						bw.write('0');
					}	
					bw.write(' ');
					String Act = Integer.toString(1);
					bw.write(Act);
					bw.newLine();
			}
			bw.close();
			fw.close();
		} catch (IOException e){
			System.out.println("Error when writing the file coaHist.txt!");
		}	
	}
}
