package outils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

//This file allows,
//Reading:
	// - the files containing the parameters: "CarryingCapacities_test.txt", "Mconnectivity_test.txt" and "DistriMainland_test.txt".
	// - for each simulation replicate SIM, the file "coaHist_struct_test_SIM.txt" = history of coalescence + 3 columns to add information when modelling speciation:
		// the resulting matrix is "input": | Time step | Localisation | Species | 0 | -1 | -1 | Event | From/Target | Position of target | number of Active individuals +1.
		// 1st, 2nd, 3rd, 7th, 8th, 9th, 10th columns = from "coaHist.txt"
		// 4th, 5th and 6th columns = filled in this algorithm = | Time (double, unit = number of time steps) | Species number (once attributed) | Clade_Num (double = clade number, when 1 individual/clade) |
//Writing:
	// - for each simulation replicate SIM, the species distribution on each archipelago (archipelago 0 + species index) given 
		// - (i) no speciation: "NoSpc_struct_test_SIM.txt" 
		// - (ii) protracted speciation:  "SpcProtr_struct_test_SIM.txt" 

public class ReadWrite {
	
	public static double [][] readInput(int archip) throws IOException{			// Reading the file "coaHist_unstruct...txt", history of coalescence; all columns which are doubles (all but the 2nd one) -> input is a table of doubles 
		BufferedReader inputRow = new BufferedReader(new FileReader("coaHist_unstruct_test_"+archip+".txt"));
		String str3 = inputRow.readLine();
		int nbRows=0;
		while(str3 != null){
			str3 = inputRow.readLine();
			nbRows++;
		}	
		//System.out.println(nbRows);
		BufferedReader mat = new BufferedReader(new FileReader("coaHist_unstruct_test_"+archip+".txt"));	
		String matrix[][];
		matrix = new String[nbRows][10];
		String str = mat.readLine();								// str = one line of the table
		int i1 = 0;
		while(str != null){							
			StringTokenizer st = new StringTokenizer(str," ");		// st = one word of the line
			matrix[i1][0] = st.nextToken();
			st.nextToken();
			matrix[i1][1] = st.nextToken();
			matrix[i1][2] = st.nextToken();
			matrix[i1][6] = st.nextToken();
			matrix[i1][7] = st.nextToken();
			matrix[i1][8] = st.nextToken();
			matrix[i1][9] = st.nextToken();
			matrix[i1][3] = "0";
			for (int k1=4; k1<6; k1++){
				matrix[i1][k1] = "-1";
			}
			str = mat.readLine();
			i1++;
		}
		double input[][];											// matrix is a table of strings: it must be transformed into a table of doubles, = input
		input = new double[nbRows][10];
		for (i1=0; i1<nbRows; i1++){
			for (int j1=0; j1<10; j1++){
				input[i1][j1] = Double.parseDouble(matrix[i1][j1]);
			}														
		}
		return input;
	}
	
	public static String [] readClade(int nbRows, int archip) throws IOException{			// Reading the file "coaHist_unstruct...txt", history of coalescence; But only columns which are strings = the 2nd one -> clade is a table of strings 
		BufferedReader mat = new BufferedReader(new FileReader("coaHist_unstruct_test_"+archip+".txt"));	
		String str = mat.readLine();
		String clade[];
		clade = new String[nbRows];
		int i1 = 0;
		while(str != null){							
			StringTokenizer st = new StringTokenizer(str," ");
			st.nextToken();
			clade[i1] = st.nextToken();
			str = mat.readLine();
			i1++;
		}
		return clade;
	}
	
	public static int[] readK() throws IOException{												// Same as in ReadTable.java in "coa_unstruct"
		BufferedReader CCPatches = new BufferedReader(new FileReader("CarryingCapacities_unstruct_test.txt")); 
		String str3 = CCPatches.readLine();					
		StringTokenizer st3 = new StringTokenizer(str3,";");		
		String nbData;
		nbData = st3.nextToken();				
		int nb = Integer.parseInt(nbData);
		String[] CCPat1 = new String[nb];
		int i3 = 0, j3 = 0;
		while(str3 != null){
			while (j3 < nb){
				CCPat1[j3] = st3.nextToken();
				j3++;
			}
			str3 = CCPatches.readLine();
		}
		int[] CCPat;									
		CCPat = new int[nb];
		for (i3=0; i3<nb; i3++){
			CCPat[i3] = Integer.parseInt(CCPat1[i3]);
		}
		return CCPat;
	}
	
	public static double [][] readConnectivity(int NM) throws IOException{						// Same as in ReadTable.java in "coa_unstruct"	
		BufferedReader connectivity = new BufferedReader(new FileReader("Mconnectivity_unstruct_test.txt"));
		String matcon[][];
		matcon = new String[NM][2];
		String str = connectivity.readLine();
		int i1 = 0, j1 = 0;
		while(str != null){
			StringTokenizer st = new StringTokenizer(str," ");
			for (j1=0; j1<2; j1++){
				matcon[i1][j1] = st.nextToken();
			}
			str = connectivity.readLine();
			i1++;
		}
		double Mcon[][];							
		Mcon = new double[NM][2];
		for (i1=0; i1<NM; i1++){
			for (j1=0; j1<2; j1++){
				Mcon[i1][j1] = Double.parseDouble(matcon[i1][j1]);
			}													
		}
		return Mcon;
	}
	
	public static int[] readDistri() throws IOException{										// Same as in ReadTable.java in "coa_unstruct"
		BufferedReader distribution = new BufferedReader(new FileReader("DistriMainland_test.txt")); 
		String str2 = distribution.readLine();					
		StringTokenizer st2 = new StringTokenizer(str2,";");		
		String nbData;
		nbData = st2.nextToken();					
		int nb = Integer.parseInt(nbData);		 
		String distri1[];
		distri1 = new String[nb];
		int i2 = 0, j2 = 0;
		while(str2 != null){
			while (j2<nb){
				distri1[j2] = st2.nextToken();
				j2++;
			}
			str2 = distribution.readLine();
		}
		int[] distri;									
		distri = new int[nb];
		for (i2=0; i2<nb; i2++){
			distri[i2] = Integer.parseInt(distri1[i2]);
		}
		return distri;
	}
	
	public static void writeNoSpeciation(double[][] input, ArrayList<Integer> firstState, int archip){			// Writing the distribution of species on each archipelago given no speciation
		try{
			FileWriter fw = new FileWriter("NoSpc_unstruct_test_"+archip+".txt",true);
			BufferedWriter bw = new BufferedWriter(fw);
			for (int i=0; i<firstState.size(); i++){
				String Loc = Double.toString(input[firstState.get(i)][1]);
				bw.write(Loc);
				bw.write(';');
				String Esp = Double.toString(input[firstState.get(i)][4]);
				bw.write(Esp);
				bw.newLine();
			}
			bw.write(' ');
			bw.write(';');
			bw.write(' ');
			bw.newLine();
			bw.close();
			fw.close();
		} catch (IOException e){
			System.out.println("Error when writing the file NoSpc_...txt!");
		}
	}
	
	public static void writeSpcProtracted(double[] Position,double[] Espece, int archip){						// Writing the distribution of species on each archipelago given protracted speciation 
		try{
			FileWriter fw = new FileWriter("SpcProtr_unstruct_test_"+archip+".txt",true);
			BufferedWriter bw = new BufferedWriter(fw);
			for (int i=0; i<Espece.length; i++){
				String Loc = Double.toString(Position[i]);
				bw.write(Loc);
				bw.write(';');
				String Esp = Double.toString(Espece[i]);
				bw.write(Esp);
				bw.newLine();
			}
			bw.write(' ');
			bw.write(';');
			bw.write(' ');
			bw.newLine();
			bw.close();
			fw.close();
		} catch (IOException e){
			System.out.println("Error when writing the file SpcProtr_...txt!");
		}
	}	
}
